import tornado.web
import tornado.escape


class optionModule(tornado.web.UIModule):
    def render(self, option):
        '''
        render option in template
        :param option:
        :return:
        '''
        from tjango.conf.models import configOption
        from tjango.db import database
        flag = False
        try:
            database.connect()
        except BaseException:
            flag = True
            pass
        try:
            if configOption.select().where(configOption.name == option).exists():
                if not flag:
                    database.close()
                return tornado.escape.xhtml_escape(
                    configOption.get(configOption.name == option).value)
            else:
                if not flag:
                    database.close()
                return ''
        except Exception as e:
            return ''




class appModelModule(tornado.web.UIModule):
    def get_template_path(self):
        """Override to customize template path for each handler.

        By default, we use the ``template_path`` application setting.
        Return None to load templates relative to the calling file.
        """
        return None

    def render(self, *args, **kwargs):
        from tjango.contrib.admin.utils import modelFinder
        modelsFinder = modelFinder()
        # a tuple containing three element : app,className,class
        modelsList = modelsFinder.getInstalledModel()
        args = locals()
        args.pop('self')
        return self.render_string(
            'templates/contrib/admin/leftSideBar.html', **args)



from tjango.conf import setting


class modelFinder:
    modelsList = []

    def __init__(self):
        '''
        read configuration then find installed_APP
        :return:
        '''
        Setting = setting()
        Setting._setup()
        # empty this ...
        self.modelsList = []
        self.install_app = Setting._wrapped.INSTALLED_APPS

    def getInstalledModel(self):
        '''
        get all install models
        :return: a list that contains some tuples formatting (name,models)
        '''
        from importlib import import_module
        import inspect
        from tjango.db.models import baseModel

        for app in self.install_app:
            # assembly the models's path
            modelsPath = app + '.models'
            modelsModule = import_module(modelsPath)
            # traverse all models
            appModels = []
            for name, model in inspect.getmembers(modelsModule):
                if inspect.isclass(model) and issubclass(
                        model, baseModel) and model != baseModel:
                    appModels.append((name, model))

            self.modelsList.append((app, appModels))
        return self.modelsList



from peewee import *
from tjango.db import database

class baseModel(Model):
    '''
    use for models
    '''
    class Meta:
        '''
        configure for database
        '''
        database = database


from __future__ import absolute_import, division, print_function
from tjango.utils.version import get_version
# version is a human-readable version number.

# version_info is a four-tuple for programmatic comparison. The first
# three numbers are the components of the version number.  The fourth
# is zero for an official release, positive for a development branch,
# or negative for a release candidate or beta (after the base version
# number has been incremented)
VERSION = (0, 1, 1, 'beta', 1)

__version__ = get_version(VERSION)


__author__ = 'kidozh'
# -*- coding: UTF-8 -*-
from tjango.conf import setting

class modelFinder:
    modelsList = []
    def __init__(self):
        '''
        read configuration then find installed_APP
        :return:
        '''
        Setting = setting()
        Setting._setup()
        # empty this ...
        self.modelsList = []
        self.install_app = Setting._wrapped.INSTALLED_APPS

    def getInstalledModel(self):
        '''
        get all install model
        :return: a list that contains some tuples formatting (name,model)
        '''
        from importlib import import_module
        import inspect
        from tjango.db.models import baseModel
        #

        for app in self.install_app:
            # assembly the model's path
            modelsPath = app + '.models'
            modelsModule = import_module(modelsPath)
            # traverse all models
            appModels = []
            for name,model in inspect.getmembers(modelsModule):
                if inspect.isclass(model) and issubclass(model,baseModel) and model != baseModel:
                    appModels.append((name,model))

            self.modelsList.append((app,appModels))
        return self.modelsList

if __name__ == '__main__':
    import os
    from tjango.conf import ENVIRONMENT_VARIABLE
    # refer the setting modules
    os.environ[ENVIRONMENT_VARIABLE] = 'settings'
    print(os.getcwd())

    finder = modelFinder()
    print(finder.getInstalledModel())

    # create all table
    from tjango.db import database

    database.connect()

    for app,modelList in finder.getInstalledModel():
        print(app,modelList)
        for name,model in modelList:
            print(name)
            try:
                database.create_tables([model])
                print(app+' @ '+ name)
            except Exception as e:
                print(e)
    database.close()


# -*- coding: UTF-8 -*-
import tornado.web
import tornado
from tjango.db import connectHandler, database
from tjango.contrib.admin.models import *
import datetime


class baseHandler(tornado.web.RequestHandler):
    def get_template_path(self):
        """Override to customize template path for each handler.

        By default, we use the ``template_path`` application setting.
        Return None to load templates relative to the calling file.
        """
        return None

    # peewee
    def prepare(self):
        try:
            database.connect()
        except BaseException:
            pass
        return super(baseHandler, self).prepare()

    def on_finish(self):
        # save to database...

        if not database.is_closed():
            database.close()
        return super(baseHandler, self).on_finish()

    def get_current_user(self):
        # check current user privilege
        username = self.get_secure_cookie("username")
        # init database connect

        # queryObj = connectInstance.query(adminUser).filter(adminUser.username == username)
        if admin.select().where(admin.username == username).exists():
            # there is a username
            aimUser = admin.select().where(admin.username == username).get()
            if aimUser.isStaff:
                # successfully
                self._current_user = aimUser
                return aimUser

        return None

    def write_error(self, status_code, **kwargs):
        args = locals()
        args.pop('self')
        self.render('error.html', **args)



# -*- coding: UTF-8 -*-
'''
    Brief monitoring module for VPS
'''

import psutil
import platform


class PsutilStats(object):

    def loadPlatform(self):
        x = platform.uname()
        return x

    def loadSummaryData(self):
        summary_data = {}
        summary_data['cpu'] = psutil.cpu_percent(interval=None, percpu=False)
        summary_data['memory'] = psutil.virtual_memory().percent
        summary_data['disk'] = psutil.disk_usage('/').percent
        summary_data['virtual'] = psutil.virtual_memory()
        summary_data['net_sent'] = psutil.net_io_counters(
            pernic=False).bytes_sent
        summary_data['net_recv'] = psutil.net_io_counters(
            pernic=False).bytes_recv
        return summary_data

    def getTopProcesses(self, limit=None):
        m = {}
        for x in psutil.get_process_list():
            cpu_time = x.get_cpu_times()[1]
            if not cpu_time:
                continue
            k = x.pid; v = {"pid": x.pid, "name": x.name, "cpu_time": cpu_time}
            m[k] = v
        p = m.items()
        p.sort(key=lambda x: x[1]["cpu_time"], reverse=True)
        return p[0:limit]

    def getProcessDetails(self, pid):
        x = psutil.Process(pid)
        xcuse = """Information about this process is either
        or you lack the proper permissions to view it"""

        try:
            process_details = {}
            process_details['Status'] = str(x.status)
            process_details['Username'] = x.username
            process_details['Terminal'] = x.terminal
            process_details['IO Count'] = x.get_io_counters()
            process_details['CPU Times'] = x.get_cpu_times()
            process_details['Threads'] = x.get_threads()
            process_details['Memory'] = x.get_memory_info()
            process_details['Connections'] = x.get_connections()
        except BaseException:
            process_details = xcuse
        # return x
        return process_details



# -*- coding: UTF-8 -*-
__author__ = 'kidozh'
"""
Default tjango settings. Override these with settings in the module pointed to
by the TJANGO_SETTINGS_MODULE environment variable.
"""

####################
# CORE             #
####################


DEBUG = False

# People who get code error notifications.
# In the format [('Full Name', 'email@example.com'), ('Full Name',
# 'anotheremail@example.com')]
ADMINS = []

# Local time zone for this installation. All choices can be found here:
# https://en.wikipedia.org/wiki/List_of_tz_zones_by_name (although not all
# systems may support all possibilities). When USE_TZ is True, this is
# interpreted as the default user time zone.
TIME_ZONE = 'Asia/Shanghai'

# If you set this to True, Tjango will use timezone-aware datetimes.
USE_TZ = False

# Host for sending email.
EMAIL_HOST = 'localhost'

# Port for sending email.
EMAIL_PORT = 25

# Whether to send SMTP 'Date' header in the local time zone or in UTC.
EMAIL_USE_LOCALTIME = False

# A secret key for this particular Django installation. Used in secret-key
# hashing algorithms. Set this in your settings, or Tjango will complain
# loudly.
SECRET_KEY = ''

# Optional SMTP authentication information for EMAIL_HOST.
EMAIL_HOST_USER = ''
EMAIL_HOST_PASSWORD = ''
EMAIL_USE_TLS = False
EMAIL_USE_SSL = False
EMAIL_SSL_CERTFILE = None
EMAIL_SSL_KEYFILE = None
EMAIL_TIMEOUT = None

# Template path configuration
DISTRIBUTE_TEMPLATE_DIR = False



# /usr/bin/python
# -*- coding: utf-8 -*-
#   格式：\033[显示方式;前景色;背景色m
#   说明:
#
#   前景色            背景色            颜色
#   ---------------------------------------
#     30                40              黑色
#     31                41              红色
#     32                42              绿色
#     33                43              黃色
#     34                44              蓝色
#     35                45              紫红色
#     36                46              青蓝色
#     37                47              白色
#
#   显示方式           意义
#   -------------------------
#      0           终端默认设置
#      1             高亮显示
#      4            使用下划线
#      5              闪烁
#      7             反白显示
#      8              不可见
#
#   例子：
#   \033[1;31;40m    <!--1-高亮显示 31-前景色红色  40-背景色黑色-->
#   \033[0m          <!--采用终端默认设置，即取消颜色设置-->]]]
STYLE = {
    'fore': {
        # 前景色
        'black': 30,
        # 黑色
        'red': 31,
        # 红色
        'green': 32,
        # 绿色
        'yellow': 33,
        # 黄色
        'blue': 34,
        # 蓝色
        'purple': 35,
        # 紫红色
        'cyan': 36,
        # 青蓝色
        'white': 37,
        # 白色
    },
    'back': {
        # 背景
        'black': 40,
        # 黑色
        'red': 41,
        # 红色
        'green': 42,
        # 绿色
        'yellow': 43,
        # 黄色
        'blue': 44,
        # 蓝色
        'purple': 45,
        # 紫红色
        'cyan': 46,
        # 青蓝色
        'white': 47,
        # 白色
    },
    'mode': {
        # 显示模式
        'mormal': 0,
        # 终端默认设置
        'bold': 1,
        # 高亮显示
        'underline': 4,
        # 使用下划线
        'blink': 5,
        # 闪烁
        'invert': 7,
        # 反白显示
        'hide': 8,
        # 不可见
    },
    'default': {'end': 0, },
}



def use_style(string, mode='', fore='', back=''):
    mode = '%s' % STYLE['mode'][mode] if STYLE['mode'].__contains__(
        mode) else ''
    fore = '%s' % STYLE['fore'][fore] if STYLE['fore'].__contains__(
        fore) else ''
    back = '%s' % STYLE['back'][back] if STYLE['back'].__contains__(
        back) else ''
    style = ';'.join([s for s in [mode, fore, back] if s])
    style = '\033[%sm' % style if style else ''
    end = '\033[%sm' % STYLE['default']['end'] if style else ''
    return '%s%s%s' % (style, string, end)



from tjango.db.models import baseModel
from peewee import *


class configOption(baseModel):
    '''
    this is option for configuration in template rendering and data rewrite like wp_option
    '''

    name = CharField(100,unique=True)
    value = CharField(100,null=True)



# -*- coding: UTF-8 -*-
import datetime
import bcrypt
from peewee import *
from tjango.db.models import baseModel
from playhouse.fields import PasswordField


class admin(baseModel):
    username = CharField(unique=True)
    nickname = CharField(max_length=20, null=True)
    password = PasswordField()

    # time
    register_time = DateTimeField(
        default=datetime.datetime.now,
        help_text=u'注册时间')

    # permissions
    isStaff = BooleanField(default=False)
    isAdmin = BooleanField(default=False)

    def authPassword(self, password):
        return self.password.check_password(password)
        # return bcrypt.checkpw(
        #     password.encode('utf-8'),
        #     self.password) and len(password) > 8



import tjango.contrib.admin.ui_module
import tornado.web
import os.path
import tjango

urlpatterns = [
    ('/', 'tjango.contrib.admin.views.adminManageHandler'),
    ('/login/', 'tjango.contrib.admin.views.authRequestHandler'),
    ('/manage/', 'tjango.contrib.admin.views.adminManageHandler'),
    ('/manage/(?P<modelName>.+)/data/', 'tjango.contrib.admin.views.appModelApiManager'),
    ('/manage/(?P<modelName>.+)/show/', 'tjango.contrib.admin.views.appModelManager'),
    ('/manage/(?P<modelName>.+)/add/', 'tjango.contrib.admin.views.appModelAddManager'),
    ('/manage/(?P<modelName>.+)/change/(?P<id>.+)/',
     'tjango.contrib.admin.views.appModelChangeManager'),
    ('/manage/(?P<modelName>.+)/delete/(?P<id>.+)/',
     'tjango.contrib.admin.views.appModelDeleteManager'),
    # monitor system
    ('/serverStatus', 'tjango.contrib.admin.views.statusAPIHandler'),
    ('/serverStatusWS', 'tjango.contrib.admin.views.statusWebsocketAPIHandler'),
    # chat channel so that administrator could chat realtime or record some
    # data
    # static admin file
    ('/static/admin/(.*?)',tornado.web.StaticFileHandler,dict(path=os.path.join(tjango.__path__[0],'contrib','admin','static')))

]

UImodule = {
    'get_option': tjango.contrib.admin.ui_module.optionModule,
    'adminlte_load_left_sidebar': tjango.contrib.admin.ui_module.appModelModule
}


# cron should be (func,every ms)

cron = [

]



import random
import hashlib
import time
from tjango.conf import setting

# Use the system PRNG if possible
try:
    random = random.SystemRandom()
    using_sysrandom = True
except NotImplementedError:
    import warnings
    warnings.warn('A secure pseudo-random number generator is not available '
                  'on your system. Falling back to Mersenne Twister.')
    using_sysrandom = False


def get_random_string(length=12,
                      allowed_chars='abcdefghijklmnopqrstuvwxyz'
                                    'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'):
    """
    Return a securely generated random string.
    The default length of 12 with the a-z, A-Z, 0-9 character set returns
    a 71-bit value. log_2((26+26+10)^12) =~ 71 bits
    """
    if not using_sysrandom:
        # This is ugly, and a hack, but it makes things better than
        # the alternative of predictability. This re-seeds the PRNG
        # using a value that is hard for an attacker to predict, every
        # time a random string is required. This may change the
        # properties of the chosen random sequence slightly, but this
        # is better than absolute predictability.
        Setting = setting()
        Setting._setup()

        random.seed(
            hashlib.sha256(
                ('%s%s%s' %
                 (random.getstate(),
                  time.time(),
                  Setting._wrapped.SECRET_KEY)).encode()).digest())
    return ''.join(random.choice(allowed_chars) for i in range(length))



from tjango.conf import setting


class connectHandler:
    def __init__(self):
        Setting = setting()
        Setting._setup()
        self.settingMapper = Setting._wrapped
        # configuration from setting
        # get configuration from setting
        # according to peewee we should form that address
        # dialect[+driver]://user:password@host/dbname
        self.DB_HOST = Setting._wrapped.DB_HOST
        self.DB_USER = Setting._wrapped.DB_USER
        self.DB_PASSWD = Setting._wrapped.DB_PASSWD
        self.DB_TYPE = Setting._wrapped.DB_TYPE
        self.DB_DRIVER = Setting._wrapped.DB_DRIVER
        self.DB_NAME = Setting._wrapped.DB_NAME

    @property
    def connect_address(self):

        # mysql
        if self.DB_TYPE == 'sqlite':
            sqlAddress = 'sqlite:///%s'%(self.DB_NAME)
        elif self.DB_DRIVER == '':
            # use default driver
            # use default utf8 to prevent encode characters error
            sqlAddress = '%s://%s:%s@%s/%s?charset=utf8' % (
                self.DB_TYPE, self.DB_USER, self.DB_PASSWD, self.DB_HOST, self.DB_NAME)
        else:
            sqlAddress = '%s+%s://%s:%s@%s/%s?charset=utf8' % (
                self.DB_TYPE, self.DB_DRIVER, self.DB_USER, self.DB_PASSWD, self.DB_HOST, self.DB_NAME)
        return sqlAddress


from tjango.db import connectHandler

connectDB = connectHandler()

from playhouse.db_url import connect
database = connect(connectDB.connect_address)



from . import baseCommand
import sys

class command(baseCommand):

    command_text = 'startproject'
    create_type = 'project'
    help_text = 'type <project_name>, <project_name> should be a string and not contradictory with existing package'

    def __init__(self):
        pass

    def execute(self):
        argv = sys.argv[1:]
        if len(argv) == 2:
            project_name = argv[1]
        else:
            raise ValueError('Only one parameters is needed, but you gave %s'%(argv))
        self.create_project_or_app(self.create_type,project_name)
        print('Project %s has been successfully created' %(project_name))


import functools
from tjango import __version__
import sys
from .color import use_style
import argparse
import importlib


def show_tjango_version():
    print(
        '-' *
        10 +
        use_style(
            ' Tjango ' +
            __version__ +
            ' ',
            fore='blue') +
        '-' *
        10)


def show_help_info():
    help_str = '''
Available subcommands:

''' + use_style('[Tjango]', fore='yellow') + '''
    startproject

    '''
    print(help_str)



def execute_from_command_line():
    """Run a ManagementUtility."""
    argv = sys.argv[1:]
    show_tjango_version()
    if len(argv) == 0:
        show_help_info()
        return

    # print(argv, sys.argv)
    module_name = argv[0]
    command_module = importlib.import_module('tjango.management.command.%s'%(module_name),'command')
    # print(command_module)
    # execute command
    command_instance = command_module.command()
    command_instance.execute()
    pass



import shutil
import os
import os.path
import tjango

class DirectoryExistError(Exception):
    pass

class baseCommand(object):

    command_text = ''
    help_text = ''

    def __init__(self):
        pass


    def create_project_or_app(self,create_type,project_or_app_name):
        if create_type not in ['project','app']:
            raise ValueError('%s you input is not supported in creating function'%(create_type))

        current_exe_path = os.getcwd()
        create_project_path = os.path.join(current_exe_path,project_or_app_name)
        # print(create_project_path,os.path.exists(create_project_path))

        # ensure there is no directory
        if os.path.exists(create_project_path):
            raise DirectoryExistError('Project %s you will create exists under %s'%(project_or_app_name,create_project_path))

        # ensure projects or app template path
        tjango_package_path = tjango.__path__[0]
        template_directory_name = '%s_template'%(create_type)
        template_directory_path = os.path.join(tjango_package_path,'conf',template_directory_name)
        # copy directory
        shutil.copytree(template_directory_path, create_project_path)
        # render the file
        from tjango.utils.crypto import get_random_string
        render_argv = dict(
            project_name = project_or_app_name,
            app_name = project_or_app_name,
            version = __import__('tjango').get_version(),
            secret_key = get_random_string(length=20)
        )
        # traverse the directory
        import tornado.template
        import tornado.escape

        for file_name in os.listdir(create_project_path):
            # only render file endswith `py-tpl`
            if not file_name.endswith('.py-tpl'):
                continue
            rendering_file_path = os.path.join(create_project_path,file_name)
            with open(rendering_file_path,'r') as rendering_file_handler:
                rendering_file_content = str(rendering_file_handler.read())
                # print(rendering_file_content)
                # using tornado template engine
            template_engine = tornado.template.Template(rendering_file_content,autoescape=None)
            new_file_name = '%s.py'%(file_name.split('.')[0])
            new_file_path = os.path.join(create_project_path,new_file_name)
            with open(new_file_path,'w') as new_file_handler:
                rendered_content = str(template_engine.generate(**render_argv),'utf-8')
                # print(rendered_content)
                new_file_handler.write(rendered_content)
            # remove old template file
            os.remove(rendering_file_path)

        pass


# for template output
import tornado.template
import os
template_path = os.path.join(os.path.dirname(__file__), "templates")
template_loader = tornado.template.Loader(template_path)

# init configuration then get connect key
from tjango.db import connectHandler

connect = connectHandler()


from setuptools import setup, find_packages

version = __import__('tjango').get_version()


EXCLUDE_FROM_PACKAGES = ['tjango.conf.project_template',
                         'tjango.conf.app_template',
                         ]

# setup for tjango
setup(
    name='tjango',
    version=version,
    keywords='admin',
    description='A package to develop tornado in django style',
    license='MIT License',
    author='Kido Zhang',
    author_email='kidozh@kidozh.com',
    packages=find_packages(),
    include_package_data=True,
    platforms='any',
    url='https://github.com/kidozh/tjango',
    install_requires=['tornado', 'peewee', 'psutil', 'bcrypt', 'peewee_migrate'],

    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],

    # interface to tjango
    entry_points={
        'console_scripts': [
            'tjango-admin = tjango.management.console_command:execute_from_command_line',
        ],
    },
)



from tjango.conf import setting
class modelFinder:
    modelsList = []
    def __init__(self):
        '''
        read configuration then find installed_APP
        :return:
        '''
        Setting = setting()
        Setting._setup()
        # empty this ...
        self.modelsList = []
        self.install_app = Setting._wrapped.INSTALLED_APPS

    def getInstalledModel(self):
        from importlib import import_module
        import inspect
        from tjango.db.models import baseModel
        #

        for app in self.install_app:
            # assembly the models's path
            modelsPath = app + '.models'
            modelsModule = import_module(modelsPath)
            # traverse all models
            appModels = []
            for name,model in inspect.getmembers(modelsModule):
                if inspect.isclass(model) and issubclass(model,baseModel) and model != baseModel:
                    appModels.append((name,model))

            self.modelsList.append((app,appModels))
        return self.modelsList

class fieldFinder:
    def __init__(self):
        pass

    def getAllField(self,appModel):
        '''
        return all the valid field
        :param appModel: a list that containing tuples like (attrname,attr)
        :return:
        '''
        from tjango.db.models import baseModel
        import peewee
        import playhouse

        attrList = []

        # traverse all the attributes
        if issubclass(appModel,baseModel) and appModel !=baseModel:
            for attrName in dir(appModel):
                attr = getattr(appModel,attrName)
                # print appModel, ' > ', attr, type(attr)
                if attr in dir(peewee) or attr in dir(playhouse):
                    # print valid filed in peewee or playhouse module
                    attrList.append((attrName,attr))
            return attrList
        else:
            raise TypeError('%s should be subclass of baseModel' % appModel )


# -*- coding: UTF-8 -*-
__author__ = 'kidozh'

import tornado.web
import tornado
import tornado.websocket
import tornado.locale
from tjango.db import connectHandler, database
from tjango.contrib.admin.models import *
import tjango
from tjango.contrib.admin.utils import modelFinder
from tjango.db.utils import fieldFinder
from tornado.web import StaticFileHandler
import os.path
import tornado.locale

# locale
user_locale = tornado.locale.get()

class authBaseHandler(tornado.web.RequestHandler):
    # do not use it in other page since it will reconfigure template

    # compatible with admin static file
    def static_url(self, path, include_host=None, **kwargs):
        if include_host is None:
            include_host = getattr(self, "include_host", False)

        if include_host:
            base = self.request.protocol + "://" + self.request.host
        else:
            base = ""

        return base +'%s'%(self.reverse_url("<class 'tornado.web.StaticFileHandler'>",path))

    def get_template_path(self):
        """Override to customize template path for each handler.

        By default, we use the ``template_path`` application setting.
        Return None to load templates relative to the calling file.
        """
        return None


    def prepare(self):
        # configuration for admin page

        # locale settings
        tornado.locale.load_gettext_translations(os.path.join(tjango.__path__[0],'conf','locale'),'tjango')

        try:
            database.connect()
        except BaseException:
            pass
        return super(authBaseHandler, self).prepare()

    def on_finish(self):
        if not database.is_closed():
            database.close()
        return super(authBaseHandler, self).on_finish()

    def get_current_user(self):
        # check current user privilege
        username = self.get_secure_cookie("username")
        # init database connect

        # queryObj = connectInstance.query(adminUser).filter(adminUser.username == username)
        if admin.select().where(admin.username == username).exists():
            # there is a username
            aimUser = admin.select().where(admin.username == username).get()
            if aimUser.isStaff:
                # successfully
                self._current_user = aimUser
                return aimUser

        return None

    def write_error(self, status_code, **kwargs):
        args = locals()
        args.pop('self')
        self.render('templates/contrib/admin/error.html', **args)


class authRequestHandler(authBaseHandler):
    def get(self, *args, **kwargs):
        # self.render('portal/index.html')

        self.render('templates/contrib/admin/login.html')

    def post(self, *args, **kwargs):
        username = self.get_argument('username')
        password = self.get_argument('password')
        self.username = username
        self.password = password

        # make sure bcrypt could encrypt this password
        if len(password) < 8 or len(password) >= 21:
            self._reason = '密码应为8-20位'
            self.write_error(500)
        else:
            # auth
            # rewrite by peewee
            # queryObj = connectInstance.query(adminUser).filter(adminUser.username == username)
            # print
            # connectInstance.query(adminUser).filter_by(username=username).count()

            # using peewee
            if admin.select().where(admin.username == username).count():
                # username exist
                aimAdmin = admin.select().where(admin.username == username).get()

                # auth password
                if aimAdmin.authPassword(password):
                    # successfully login
                    self.set_secure_cookie('username', username)
                    # URL configuration
                    from tjango.conf import setting
                    from tjango.conf.urls import urlPackage
                    Setting = setting()
                    Setting._setup()
                    urlMapper = urlPackage(Setting._wrapped.ROOT_URLCONF)
                    # check permission
                    # auth permission

                    if not aimAdmin.isStaff:
                        self._reason = '您的账号目前无法访问仪表盘，请联系管理员。'
                        self.write_error(404)
                    # redirect
                    self.redirect(
                        self.reverse_url('tjango.contrib.admin.views.adminManageHandler'))
                else:
                    self._reason = user_locale.translate('Wrong username or password')
                    self.write_error(404)
            else:
                self._reason = user_locale.translate('Wrong username or password')
                self.write_error(404)

            args = locals()
            args.pop('self')
            # self.render('contrib/admin/login.html', **args)

    def write_error(self, status_code, **kwargs):
        args = locals()
        args.pop('self')
        self.render('templates/contrib/admin/loginError.html', **args)


class adminManageHandler(authBaseHandler):
    @tornado.web.authenticated
    def get(self, *args, **kwargs):
        import psutil
        import platform
        # cpu info
        logicalCPU = psutil.cpu_count(logical=True)
        physicsCPU = psutil.cpu_count(logical=False)

        # platform uname
        (system, node, release, version, machine, processor) = platform.uname()

        # cpu usage
        usedCPURate = psutil.cpu_percent(interval=1, percpu=False)

        # memory info
        mem = psutil.virtual_memory()

        # start time
        startTime = psutil.boot_time()

        # network statistics
        network = psutil.net_io_counters(pernic=False)

        from tjango.conf import setting
        Setting = setting()
        Setting._setup()

        logFileExist = getattr(Setting._wrapped, 'LOGFILE', False)

        modelsFinder = modelFinder()
        # a tuple containing three element : app,className,class
        modelsList = modelsFinder.getInstalledModel()

        # fill render parameter
        args = locals()
        args.pop('self')
        self.render('templates/contrib/admin/manage.html', **args)


# a HTTP handler for status api
class statusAPIHandler(authBaseHandler):
    @tornado.web.authenticated
    def get(self, *args, **kwargs):
        # import module for neccessary
        import tjango.contrib.admin.system_status
        import tornado.escape

        sysInfo = tjango.contrib.admin.system_status.PsutilStats()
        infoDict = sysInfo.loadSummaryData()
        jsonString = tornado.escape.json_encode(infoDict)
        self.write(jsonString)
        self.finish()


# advanced websocket server
class statusWebsocketAPIHandler(tornado.websocket.WebSocketHandler):
    '''
    this socketHandler is aimed to reduce stress from query system's status
    '''
    clients = []
    cache_size = 200

    # you should change it by yourself
    def check_origin(self, origin):
        return True

    def open(self, *args, **kwargs):
        '''
        make auth then add to client
        :param args:
        :param kwargs:
        :return:
        '''
        import tjango.contrib.admin.system_status
        import tornado.escape

        sysInfo = tjango.contrib.admin.system_status.PsutilStats()
        infoDict = sysInfo.loadSummaryData()
        jsonString = tornado.escape.json_encode(infoDict)
        self.write_message(jsonString)
        # directly get info
        self.clients.append(self)

    def on_message(self, message):

        import tjango.contrib.admin.system_status
        import tornado.escape

        sysInfo = tjango.contrib.admin.system_status.PsutilStats()
        infoDict = sysInfo.loadSummaryData()
        jsonString = tornado.escape.json_encode(infoDict)
        self.write_message(jsonString)

    def on_close(self):
        # msgDict = {'result': 3, 'response-Text': 'GoodBye~', 'response-Time': ''}
        # 0 : connect start
        # 1 : right ,give answer
        # 2 : wrong , give reason
        # 3 : shutdown websocket
        # response Text
        # self.write_message(json.dumps(msgDict))
        for client in self.clients:
            if client == self:
                self.clients.remove(client)
            else:
                pass

    @classmethod
    def update_cache(cls):
        # cls.cache.append(chat)
        if len(cls.cache) > cls.cache_size:
            cls.cache = cls.cache[-cls.cache_size:]


class appModelManager(authBaseHandler):
    def getModelByName(self, modelName):
        '''
        get models by models's name
        :param modelName: models class
        :return:models class or None object
        '''
        modelFinderInstance = modelFinder()
        packageModels = modelFinderInstance.getInstalledModel()
        # traverse all list then find it

        for package, list in packageModels:
            for name, model in list:
                if name == modelName:
                    return package, model
        return None

    def getValidFieldByModel(self, model):
        '''
        get all valid field in peewee or playhouse
        :param model: models instance which inherit baseModel
        :return: all valid field in peewee or playhouse
        '''
        testFiledFinder = fieldFinder()
        return testFiledFinder.getAllField(model)

    @tornado.web.authenticated
    def get(self, modelName, *args, **kwargs):
        package, aimModel = self.getModelByName(modelName)
        if aimModel is None:
            self._reason = '您所请求的模型不存在或者并未被注册'
            self._status_code = 404
            self.write_error(404)
            return None
        # show this models
        # get all valid field
        allValidField = self.getValidFieldByModel(aimModel)
        allFieldName = [getattr(aimModel, foo)
                        for (foo, instance) in allValidField]
        # set up query
        total_record_cnt = aimModel.select().count()
        query = (aimModel
                 .select(*allFieldName)
                 )
        # for foo in query:
        #    for attrName in [f o for (fo,instance) in allValidField]:
        #        print attrName,getattr(foo,attrName)

        modelsFinder = modelFinder()
        # a tuple containing three element : app,className,class
        modelsList = modelsFinder.getInstalledModel()

        args = locals()
        args.pop('self')
        self.render('templates/contrib/admin/tableData.html', **args)


class appModelAddManager(authBaseHandler):
    def getValidFieldByModel(self, model):
        '''
        get all valid field in peewee or playhouse
        :param model: models instance which inherit baseModel
        :return: all valid field in peewee or playhouse
        '''
        from tjango.db.utils import fieldFinder
        testFiledFinder = fieldFinder()
        return testFiledFinder.getAllField(model)

    def getModelByName(self, modelName):
        '''
        get models by models's name
        :param modelName: models class
        :return:models class or None object
        '''
        modelFinderInstance = modelFinder()
        packageModels = modelFinderInstance.getInstalledModel()
        # traverse all list then find it

        for package, list in packageModels:
            for name, model in list:
                if name == modelName:
                    return package, model
        return None

    def get(self, modelName, *args, **kwargs):
        package, aimModel = self.getModelByName(modelName)
        if aimModel is None:
            self._reason = '您所请求的模型不存在或者并未被注册'
            self._status_code = 404
            self.write_error(404)
            return None

        allValidField = self.getValidFieldByModel(aimModel)
        allFieldName = [getattr(aimModel, fieldName)
                        for (fieldName, field) in allValidField]

        args = locals()
        args.pop('self')
        self.render('templates/contrib/admin/addData.html', **args)

    def post(self, modelName, *args, **kwargs):
        package, aimModel = self.getModelByName(modelName)
        if aimModel is None:
            self._reason = '您所请求的模型不存在或者并未被注册'
            self._status_code = 404
            self.write_error(404)
            return None

        allValidField = self.getValidFieldByModel(aimModel)
        # initialize aimModel
        modelRec = aimModel()
        # traverse all the POST parameters

        try:
            for key in self.request.arguments:
                # print key,self.get_argument(key,None)
                if hasattr(modelRec, key) and self.get_argument(key, 0):
                    # print
                    # key,self.get_argument(key),type(getattr(modelRec,key)) ==
                    # bool
                    if isinstance(
                            getattr(
                                modelRec,
                                key),
                            bool) and self.get_argument(key) == '1':

                        setattr(modelRec, key, True)
                    elif isinstance(getattr(modelRec, key), bool) and self.get_argument(key) == '0':

                        setattr(modelRec, key, False)
                    else:
                        setattr(modelRec, key, self.get_argument(key, 0))
                else:
                    continue
            # save the record
            modelRec.save()
        except Exception as e:
            self._reason = e
            self.write_error(500)
            return
        self.redirect(
            self.reverse_url(
                'tjango.contrib.admin.views.appModelManager',
                modelName))

        args = locals()
        args.pop('self')
        self.render('templates/contrib/admin/addData.html', **args)


class appModelChangeManager(authBaseHandler):
    def getValidFieldByModel(self, model):
        '''
        get all valid field in peewee or playhouse
        :param model: models instance which inherit baseModel
        :return: all valid field in peewee or playhouse
        '''

        testFiledFinder = fieldFinder()
        return testFiledFinder.getAllField(model)

    def getModelByName(self, modelName):
        '''
        get models by models's name
        :param modelName: models class
        :return:models class or None object
        '''
        modelFinderInstance = modelFinder()
        packageModels = modelFinderInstance.getInstalledModel()
        # traverse all list then find it

        for package, list in packageModels:
            for name, model in list:
                if name == modelName:
                    return package, model
        return None

    def getPrimaryKey(self, model):
        import peewee
        fields = self.getValidFieldByModel(model)
        for name, attr in fields:
            if isinstance(attr, peewee.PrimaryKeyField):
                return attr
        return None

    @tornado.web.authenticated
    def get(self, modelName, id, *args, **kwargs):
        # just show result
        package, aimModel = self.getModelByName(modelName)
        if aimModel is None:
            self._reason = '您所请求的模型不存在或者并未被注册'
            self._status_code = 404
            self.write_error(404)
            return None

        allValidField = self.getValidFieldByModel(aimModel)
        primaryKey = self.getPrimaryKey(aimModel)
        aimRow = aimModel.get(primaryKey == id)

        args = locals()
        args.pop('self')
        self.render('templates/contrib/admin/changeData.html', **args)

    @tornado.web.authenticated
    def post(self, modelName, id, *args, **kwargs):
        import peewee
        import playhouse.fields

        package, aimModel = self.getModelByName(modelName)
        if aimModel is None:
            self._reason = '您所请求的模型不存在或者并未被注册'
            self._status_code = 404
            self.write_error(404)
            return None

        allValidField = self.getValidFieldByModel(aimModel)
        primaryKey = self.getPrimaryKey(aimModel)
        aimRow = aimModel.get(primaryKey == id)
        for key in self.request.arguments:
            if hasattr(aimRow, key) and self.get_argument(key, None):

                if isinstance(getattr(aimModel, key), peewee.BooleanField):
                    value = self.get_argument(key)
                    # print value,getattr(aimModel,key)
                    if value == '0':
                        setattr(aimRow, key, False)
                    else:
                        setattr(aimRow, key, True)
                # handle time and datetime please take attention

                else:
                    # for non-password field
                    setattr(aimRow, key, self.get_argument(key, None))
            else:
                continue
        aimRow.save()
        self.redirect(
            self.application.reverse_url(
                'tjango.contrib.admin.views.appModelManager',
                modelName))


class appModelDeleteManager(authBaseHandler):
    def getValidFieldByModel(self, model):
        '''
        get all valid field in peewee or playhouse
        :param model: models instance which inherit baseModel
        :return: all valid field in peewee or playhouse
        '''

        testFiledFinder = fieldFinder()
        return testFiledFinder.getAllField(model)

    def getModelByName(self, modelName):
        '''
        get models by models's name
        :param modelName: models class
        :return:models class or None object
        '''
        modelFinderInstance = modelFinder()
        packageModels = modelFinderInstance.getInstalledModel()
        # traverse all list then find it

        for package, list in packageModels:
            for name, model in list:
                if name == modelName:
                    return package, model
        return None

    def getPrimaryKey(self, model):
        import peewee
        fields = self.getValidFieldByModel(model)
        for name, attr in fields:
            if isinstance(attr, peewee.PrimaryKeyField):
                return attr
        return None

    @tornado.web.authenticated
    def get(self, modelName, id, *args, **kwargs):
        # just show result
        package, aimModel = self.getModelByName(modelName)
        if aimModel is None:
            self._reason = '您所请求的模型不存在或者并未被注册'
            self._status_code = 404
            self.write_error(404)
            return None

        allValidField = self.getValidFieldByModel(aimModel)
        primaryKey = self.getPrimaryKey(aimModel)
        aimRow = aimModel.get(primaryKey == id)

        args = locals()
        args.pop('self')
        self.render('templates/contrib/admin/deleteData.html', **args)

    @tornado.web.authenticated
    def post(self, modelName, id, *args, **kwargs):
        import peewee
        import playhouse.fields

        package, aimModel = self.getModelByName(modelName)
        if aimModel is None:
            self._reason = '您所请求的模型不存在或者并未被注册'
            self._status_code = 404
            self.write_error(404)
            return None

        allValidField = self.getValidFieldByModel(aimModel)
        primaryKey = self.getPrimaryKey(aimModel)
        aimRow = aimModel.get(primaryKey == id)
        # print aimRow.delete_instance()
        aimRow.delete_instance()

        self.redirect(
            self.application.reverse_url(
                'tjango.contrib.admin.view.appModelManager',
                modelName))


class appModelApiManager(authBaseHandler):
    '''
    a api server for dataTables
    '''

    def getModelByName(self, modelName):
        '''
        get models by models's name
        :param modelName: models class
        :return:models class or None object
        '''
        modelFinderInstance = modelFinder()
        packageModels = modelFinderInstance.getInstalledModel()
        # traverse all list then find it

        for package, list in packageModels:
            for name, model in list:
                if name == modelName:
                    return package, model
        return None

    def getValidFieldByModel(self, model):
        '''
        get all valid field in peewee or playhouse
        :param model: models instance which inherit baseModel
        :return: all valid field in peewee or playhouse
        '''
        testFiledFinder = fieldFinder()
        return testFiledFinder.getAllField(model)

    @tornado.web.authenticated
    def get(self, *args, **kwargs):
        self._reason = 'API目前使用的是dataTable，很抱歉目前您只能使用POST的方法来获得信息。'
        self.write_error(403)

    @tornado.web.authenticated
    def post(self, modelName, *args, **kwargs):
        # get models's name from URL directly
        package, aimModel = self.getModelByName(modelName)
        if aimModel is None:
            self._reason = '您所请求的模型不存在或者并未被注册'
            self._status_code = 404
            self.write_error(404)
            return None
        # show this models
        # get all valid field
        allValidField = self.getValidFieldByModel(aimModel)
        allFieldName = [getattr(aimModel, foo)
                        for (foo, instance) in allValidField]
        # set up query

        # this parameter will be returned directly for ajax query
        draw = self.get_argument('draw')
        start = self.get_argument('start')
        length = self.get_argument('length')

        # read columns configuration from dataTable
        i = 0
        success = True
        # traverse to get value from dataTable configuration
        # first read columns name from dataTable
        columnsInfo = []
        while success:
            # try to get parameters ,if error break right now
            # build up parameters
            parameterData = 'columns[%s][data]' % i
            parameterName = 'columnsCustome[%s][name]' % i
            parameterSearchable = 'columns[%s][searchable]' % i
            parameterOrderable = 'columns[%s][orderable]' % i
            parameterSearchVal = 'columns[%s][search][value]' % i
            parameterSearchRegex = 'columns[%s][search][regex]' % i

            try:
                # then search it
                data = self.get_argument(parameterData)
                name = self.get_argument(parameterName)
                searchable = self.get_argument(parameterSearchable)
                orderable = self.get_argument(parameterOrderable)
                searchval = self.get_argument(parameterSearchVal)
                searchRegex = self.get_argument(parameterSearchRegex)
                columnsInfo.append(
                    (data, name, searchable, orderable, searchval, searchRegex))
            except Exception as e:
                success = False
            finally:
                i += 1

        # then get order information
        orderInfo = []
        i = 0
        success = True
        while success:
            # build up order information parameters
            parameterOrderColumn = 'order[%s][column]' % i
            parameterOrderDir = 'order[%s][dir]' % i
            try:
                orderColumn = self.get_argument(parameterOrderColumn)
                orderDir = self.get_argument(parameterOrderDir)
                orderInfo.append((orderColumn, orderDir))
            except BaseException:
                success = False
            finally:
                i += 1

        # build up query
        # get needed paramter from Database

        allNeededField = [
            getattr(
                aimModel,
                name) for (
                data,
                name,
                searchable,
                orderable,
                searchVal,
                searchRegex) in columnsInfo]
        # get orderable info from parameter
        orderField = []

        for (orderCol, orderDir) in orderInfo:

            orderCol = int(orderCol)
            aimOrder = getattr(aimModel, columnsInfo[orderCol][1])
            # check whether this columns are orderable
            if columnsInfo[orderCol][3] == False:
                break
            if orderDir == 'desc':
                orderField.append(aimOrder.desc())
            else:
                orderField.append(aimOrder)
        # get paramter from where
        # assembly query
        recordsTotal = aimModel.select().count()
        start = int(start)
        length = int(length)
        if length > recordsTotal - start:
            length = recordsTotal - start
        searchVal = self.get_argument('search[value]')

        # print '#',searchval,self.get_argument('search[value]')

        if searchVal:
            # find the searchable columns
            # print '# search Value : ',searchVal

            searchRequirement = [
                getattr(
                    aimModel,
                    name).contains(searchVal) for (
                    data,
                    name,
                    searchable,
                    orderable,
                    i,
                    searchRegex) in columnsInfo if searchable and (
                    isinstance(
                        getattr(
                            aimModel,
                            name),
                        TextField) or isinstance(
                        getattr(
                            aimModel,
                            name),
                        CharField))]
            # !!!! default is using OR operator !!!!
            # print '# ',searchRequirement
            if len(searchRequirement) > 0:
                limit = searchRequirement[0]
                for eachLimit in searchRequirement[1:]:
                    limit = limit | eachLimit
                # print
                # len(aimModel.select(*allNeededField).where(getattr(aimModel,'codeA').contains('%s'%(searchVal))))
                query = aimModel.select(
                    *
                    allNeededField).order_by(
                    *
                    orderField).where(limit)[
                    start:start +
                    length]
                recordsFiltered = aimModel.select(
                    *
                    allNeededField).order_by(
                    *
                    orderField).where(limit).count()
            else:
                query = aimModel.select(
                    *
                    allNeededField).order_by(
                    *
                    orderField)[
                    start:start +
                    length]
                recordsFiltered = aimModel.select(
                    *
                    allNeededField).order_by(
                    *
                    orderField).count()
        else:
            query = aimModel.select(
                *
                allNeededField).order_by(
                *
                orderField)[
                start:start +
                length]
            recordsFiltered = aimModel.select(
                *
                allNeededField).order_by(
                *
                orderField).count()
        # assembly the result

        # recordsFiltered = len(query)
        data = []
        import datetime
        import tornado.locale
        import time

        for row in query:
            listData = []
            dataDict = {}
            for index in range(len(columnsInfo)):
                name = columnsInfo[index][1]
                obj = getattr(row, name)
                if isinstance(obj, datetime.datetime):
                    # obj = time.mktime(obj.timetuple())
                    obj = obj.strftime("%Y-%m-%d %H:%M:%S")
                elif (isinstance(obj, str)):
                    # only add 100
                    if len(obj) > 100:
                        obj = '%s...' % (obj[0:99])
                elif isinstance(obj,bytes):
                    obj = obj.decode('utf-8')
                dataDict[name] = obj
                listData.append(obj)
            data.append(listData)

        # retrevie data from local variables
        returnedData = {}
        returnedData['draw'] = draw
        returnedData['recordsTotal'] = recordsTotal
        returnedData['recordsFiltered'] = recordsFiltered
        returnedData['data'] = data
        print(data)

        import tornado.escape
        # self.write(json.dumps(returnedData))
        self.write(tornado.escape.json_encode(returnedData))
        self.finish()



from . import baseCommand
import sys

class command(baseCommand):
    command_text = 'startapp'
    create_type = 'app'
    help_text = 'type <app_name>, <app_name> should be a string and not contradictory with existing package'

    def __init__(self):
        pass

    def execute(self):
        argv = sys.argv[1:]
        if len(argv) == 2:
            project_name = argv[1]
        else:
            raise ValueError('Only one parameters is needed, but you gave %s'%(argv))
        self.create_project_or_app(self.create_type,project_name)
        print('App %s has been successfully created' %(project_name))


__author__ = 'kidozh'
# -*- coding: UTF-8 -*-

from importlib import import_module
from tjango.exception import ImproperlyConfigured
from tjango.conf import setting


class urlPackage:
    """
    store the URL
    """
    pattern = []
    # this is the map from handler to URL
    URLMapper = {}

    def __init__(self, url_module):
        # check whether this file is properly configured
        if isinstance(url_module, str):
            # url_module is string , just import it
            urlconf_module = import_module(url_module)
        else:
            urlconf_module = url_module
        patterns = getattr(urlconf_module, 'urlpatterns', [])
        # get this
        if isinstance(patterns, list):
            self.patterns = patterns
        else:
            raise ImproperlyConfigured(
                'url_module should contain a list-typed urlpattern variable'
            )

    def detectURL(self, headURL='', itertime=0):
        """
        get url
        :headURL :the former URL which need to be merged
        :return: (url,url_module)
        """
        from tornado.web import URLSpec
        urlList = []
        # print urlList
        for it in self.patterns:
            if isinstance(it, URLSpec):
                urlList.append(it)
                continue
            else:
                # compatible with native tornado url configuration
                url, url_module = it[:2]
                if len(it) > 2:
                    # there maybe some other configuration setting
                    other_config = it[2:]
                else:
                    other_config = []

            # iterable get the URL
            if isinstance(url_module, urlPackage):
                # BFS get URL then extend it
                urlList.extend(
                    url_module.detectURL(
                        headURL='%s%s' %
                        (headURL, url), itertime=itertime + 1))
            elif isinstance(url_module, str):
                # print url_module
                # for individual one then add it to the list
                if headURL:
                    assemblyURL = '%s%s' % (headURL, url)
                else:
                    assemblyURL = '%s' % url
                    # convert it to URLSpec
                urlList.append(
                    URLSpec(
                        assemblyURL,
                        url_module,
                        *other_config,
                        name=url_module))
                # set up mapper for
                self.URLMapper[url_module] = assemblyURL
            else:
                # try to use directly
                if headURL:
                    assemblyURL = '%s%s' % (headURL, url)
                else:
                    assemblyURL = '%s' % url
                urlList.append(
                    URLSpec(
                        assemblyURL,
                        url_module,
                        *other_config
                        ,name=str(url_module)
                    )
                )
                # raise ImproperlyConfigured(
                #     'URL only accept string or URLpackage object(you can use include)'
                # )
        return urlList


def include(urlModules):
    """
    include modules then return all of their package
    """
    # get pattern from configured file

    # for configure
    return urlPackage(urlModules)


class templateModuleFinder:
    def __init__(self):
        '''
        init conf object and get searched path
        :return:
        '''
        # get conf object
        Setting = setting()
        Setting._setup()
        self.searchApp = getattr(Setting._wrapped, 'INSTALLED_APPS')

    @property
    def uiDict(self):
        if not self.searchApp:
            # return a null dict
            return {}
        else:
            returnedDict = {}
            for app in self.searchApp:
                appUrlPath = '%s.urls' % (app)
                appUrlConf = import_module(appUrlPath)
                appUIDict = getattr(appUrlConf, 'UImodule', {})
                # traverse dict and try to import it
                if isinstance(appUIDict, dict):
                    # merge dict
                    returnedDict.update(appUIDict)
                else:
                    raise ImproperlyConfigured(
                        'UImodule in %s should be configured in dict way' % (app))
            return returnedDict


class periodTaskFinder:
    def __init__(self):
        '''
        init conf object and get searched path
        :return:
        '''
        # get conf object

        Setting = setting()
        Setting._setup()
        self.searchApp = getattr(Setting._wrapped, 'INSTALLED_APPS')

    @property
    def periodTask(self):
        '''
        generate a task recursively
        :return:
        '''
        if not self.searchApp:
            # return a null dict
            return None
        else:

            returnedList = []
            for app in self.searchApp:
                appUrlPath = '%s.urls' % (app)
                appUrlConf = import_module(appUrlPath)
                appUIDict = getattr(appUrlConf, 'cron', [])
                # traverse dict and try to import it
                if isinstance(appUIDict, list):
                    # merge dict
                    returnedList.extend(appUIDict)
                else:
                    raise ImproperlyConfigured(
                        'UImodule in %s should be configured in dict way' % (app))
            return returnedList
